package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.request.SaveCourseReq;
import cn.com.qmth.examcloud.core.basic.api.response.SaveCourseResp;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Course;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatly;
import cn.com.qmth.examcloud.core.basic.service.bean.CourseAssembler;
import cn.com.qmth.examcloud.core.basic.service.bean.CourseDto;
import cn.com.qmth.examcloud.core.basic.service.impl.CourseService;
import cn.com.qmth.examcloud.core.basic.service.impl.ExportService;
import io.swagger.annotations.ApiOperation;

/**
 * 课程服务API Created by songyue on 17/1/14.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/course")
public class CourseController extends ControllerSupport {

	@Autowired
	CourseRepo courseRepo;

	@Autowired
	CourseService courseService;

	@Autowired
	CourseAssembler courseAssembler;

	@Autowired
	CourseCloudService courseCloudService;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param courseCriteria
	 * @param specialtyId
	 * @param curPage
	 * @param pageSize
	 * @param request
	 * @return
	 */
	@ApiOperation(value = "查询课程分页带查询", notes = "分页带查询")
	@GetMapping("/all/{curPage}/{pageSize}")
	public Page<Course> getCourseList(@ModelAttribute Course courseCriteria,
			@RequestParam(required = false) Long specialtyId, @PathVariable Integer curPage,
			@PathVariable Integer pageSize, HttpServletRequest request) {
		User accessUser = getAccessUser();

		Specification<Course> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("orgId"), accessUser.getRootOrgId()));

			if (!StringUtils.isEmpty(courseCriteria.getName())) {
				predicates.add(cb.like(root.get("name"), "%" + courseCriteria.getName() + "%"));
			}
			if (!StringUtils.isEmpty(courseCriteria.getCode())) {
				predicates.add(cb.like(root.get("code"), "%" + courseCriteria.getCode() + "%"));
			}
			if (!StringUtils.isEmpty(courseCriteria.getLevel())) {
				predicates.add(cb.equal(root.get("level"), courseCriteria.getLevel()));
			}
			if (!StringUtils.isEmpty(courseCriteria.getEnable())) {
				predicates.add(cb.equal(root.get("enable"), courseCriteria.getEnable()));
			}
			if (specialtyId != null) {
				Subquery<CourseSpeciatly> subquery = query.subquery(CourseSpeciatly.class);
				Root<CourseSpeciatly> subRoot = subquery.from(CourseSpeciatly.class);
				subquery.select(subRoot);
				Predicate p1 = cb.equal(subRoot.get("specialtyId"), specialtyId);
				Predicate p2 = cb.equal(subRoot.get("courseId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(curPage - 1, pageSize,
				new Sort(Direction.DESC, "updateTime"));

		Page<Course> page = courseRepo.findAll(specification, pageRequest);
		return page;
	}

	@ApiOperation(value = "按代码或名称查询课程", notes = "代码或名称查询")
	@GetMapping("/codeOrName")
	public ResponseEntity getByCodeName(@RequestParam String codeName, HttpServletRequest request) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			return new ResponseEntity(
					courseRepo.findByCodeName(accessUser.getRootOrgId(), codeName), HttpStatus.OK);
		} else {
			return new ResponseEntity(HttpStatus.NOT_FOUND);
		}
	}

	@ApiOperation(value = "查询课程不分页带查询", notes = "不分页带查询")
	@GetMapping("/all")
	public ResponseEntity getEnableExam(@ModelAttribute Course course, HttpServletRequest request) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			course.setOrgId(accessUser.getRootOrgId());
		}
		course.setEnable(true);
		return new ResponseEntity(courseService.findAll(course), HttpStatus.OK);
	}

	@ApiOperation(value = "查询课程不分页带查询", notes = "不分页带查询")
	@GetMapping("/all/both")
	public ResponseEntity getAllExam(@ModelAttribute Course course, HttpServletRequest request) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			course.setOrgId(accessUser.getRootOrgId());
		}
		return new ResponseEntity(courseService.findAll(course), HttpStatus.OK);
	}

	@ApiOperation(value = "按ID查询课程", notes = "ID查询")
	@GetMapping("/{id}")
	public ResponseEntity getCourseById(@PathVariable Long id) {
		return new ResponseEntity(courseRepo.findOne(id), HttpStatus.OK);
	}

	@ApiOperation(value = "按code查询课程", notes = "code查询")
	@GetMapping()
	public ResponseEntity getCourseByCode(@RequestParam Long orgId, @RequestParam String code) {
		return new ResponseEntity(courseRepo.findByOrgIdAndCode(orgId, code), HttpStatus.OK);
	}

	@ApiOperation(value = "按orgId查询课程", notes = "orgId查询")
	@GetMapping("/org/{orgId}")
	public ResponseEntity getCourseByOrgId(@PathVariable Long orgId) {
		return new ResponseEntity(courseRepo.findByOrgId(orgId), HttpStatus.OK);
	}

	@ApiOperation(value = "批量查询课程", notes = "批量查询课程")
	@PostMapping("/batchGetCourses")
	public ResponseEntity<List<Course>> batchGetCourses(@RequestBody String[] funcIds,
			HttpServletRequest request) {
		List<Course> list = Lists.newArrayList();
		for (String id : funcIds) {
			Course one = courseRepo.findOne(Long.parseLong(id));
			if (null != one) {
				list.add(one);
			}
		}
		return new ResponseEntity<List<Course>>(list, HttpStatus.OK);
	}

	/**
	 * 修正
	 *
	 * @author WANGWEI
	 * @param course
	 * @return
	 */
	@ApiOperation(value = "新增课程", notes = "新增")
	@PostMapping
	public Long addCourse(@RequestBody Course course) {
		trim(course);
		User accessUser = getAccessUser();

		SaveCourseReq saveCourseReq = new SaveCourseReq();
		saveCourseReq.setCourseCode(course.getCode());
		saveCourseReq.setCourseName(course.getName());
		saveCourseReq.setCourseLevel(course.getLevel().getName());
		saveCourseReq.setRootOrgId(accessUser.getRootOrgId());

		SaveCourseResp saveCourseResp = courseCloudService.saveCourse(saveCourseReq);
		CourseBean courseBean = saveCourseResp.getCourseBean();
		return courseBean.getId();
	}

	/**
	 * 修正
	 *
	 * @author WANGWEI
	 * @param course
	 * @param request
	 * @return
	 */
	@ApiOperation(value = "更新课程", notes = "更新")
	@PutMapping
	public Long updateCourse(@RequestBody Course course, HttpServletRequest request) {
		User accessUser = getAccessUser();

		SaveCourseReq saveCourseReq = new SaveCourseReq();
		saveCourseReq.setCourseCode(course.getCode());
		saveCourseReq.setCourseName(course.getName());
		saveCourseReq.setCourseLevel(course.getLevel().getName());
		saveCourseReq.setRootOrgId(accessUser.getRootOrgId());
		if (null != course.getEnable()) {
			saveCourseReq.setEnable(course.getEnable());
		}

		SaveCourseResp saveCourseResp = courseCloudService.saveCourse(saveCourseReq);
		CourseBean courseBean = saveCourseResp.getCourseBean();
		return courseBean.getId();
	}

	@ApiOperation(value = "按ID删除课程", notes = "删除")
	@DeleteMapping("/{ids}")
	public ResponseEntity deleteCourse(@PathVariable String ids) {
		courseService.delete(ids);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "导入课程", notes = "导入")
	@PostMapping("/import")
	public List<ExcelError> importCourse(HttpServletRequest request,
			@RequestParam CommonsMultipartFile file) throws IOException {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();
		List<ExcelError> errors = courseService.importCourse(rootOrgId, file.getInputStream());
		return errors;
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("/download")
	public void importFileTemplate(HttpServletResponse response) {
		List<CourseDto> list = new ArrayList<CourseDto>();
		list.add(new CourseDto("大学英语", "000001", "0"));
		ExportService.exportEXCEL("课程导入模板", CourseDto.class, list, response);
	}

	@ApiOperation(value = "导出课程", notes = "导出")
	@GetMapping("/export")
	public void exportCourse(@ModelAttribute Course orgCriteria, HttpServletResponse response) {
		List<CourseDto> list = new ArrayList<CourseDto>();
		courseService.findAll(orgCriteria).forEach(c -> {
			list.add(courseAssembler.toDTO(c));
		});
		ExportService.exportEXCEL("课程列表", CourseDto.class, list, response);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@ApiOperation(value = "禁用课程", notes = "禁用")
	@PutMapping("/disable/{ids}")
	public List<Long> disableCourse(@PathVariable String ids) {
		List<Long> courseIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		courseService.setCourseEnable(courseIds, false);
		return courseIds;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@ApiOperation(value = "启用课程", notes = "启用")
	@PutMapping("/enable/{ids}")
	public List<Long> enableCourse(@PathVariable String ids) {
		List<Long> courseIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		courseService.setCourseEnable(courseIds, true);
		return courseIds;
	}
}
