package cn.com.qmth.examcloud.core.basic.service.impl;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelReader;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Course;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatly;
import cn.com.qmth.examcloud.core.basic.dao.enums.CourseLevel;
import cn.com.qmth.examcloud.core.basic.service.bean.CourseDto;

@Service
public class CourseService {

	@Autowired
	CourseRepo courseRepo;

	@Transactional
	public List<ExcelError> importCourse(Long orgId, InputStream inputStream) {
		ExcelReader excelReader = new ExcelReader(CourseDto.class);
		List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
			@Override
			public ExcelError handle(Object obj) {
				CourseDto dto = (CourseDto) obj;
				Course course = new Course(dto.getName(), dto.getCode());
				course.setOrgId(orgId);
				course.setEnable(true);
				if ("ZSB".equalsIgnoreCase(dto.getLevel())) {
					course.setLevel(CourseLevel.ZSB);
				}
				if ("GQZ".equalsIgnoreCase(dto.getLevel())) {
					course.setLevel(CourseLevel.GQZ);
				}
				if ("GQB".equalsIgnoreCase(dto.getLevel())) {
					course.setLevel(CourseLevel.GQB);
				}
				if ("ALL".equalsIgnoreCase(dto.getLevel())) {
					course.setLevel(CourseLevel.ALL);
				}
				ExcelError error = importCheck(course);
				if (error == null) {
					addCourse(course);
				}
				return error;
			}
		});
		return excelErrors;
	}

	private void addCourse(Course course) {
		Course old = courseRepo.findByOrgIdAndCode(course.getOrgId(), course.getCode());
		if (old == null) {
			courseRepo.save(course);
		} else {
			old.setName(course.getName());
			old.setLevel(course.getLevel());
			old.setUpdateTime(new Date());
			courseRepo.save(old);
		}
	}

	private ExcelError importCheck(Course course) {
		if (StringUtils.isEmpty(course.getCode())) {
			return new ExcelError("代码不能为空");
		}
		if (StringUtils.isEmpty(course.getName())) {
			return new ExcelError("名称不能为空");
		}
		if (null == course.getLevel()) {
			return new ExcelError("层次不能为空");
		}
		return null;
	}

	private Specification<Course> getSpecification(Course courseCriteria, Long specialtyId) {
		Specification<Course> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (!StringUtils.isEmpty(courseCriteria.getOrgId())) {
				predicates.add(cb.equal(root.get("orgId"), courseCriteria.getOrgId()));
			}
			if (!StringUtils.isEmpty(courseCriteria.getName())) {
				predicates.add(cb.like(root.get("name"), "%" + courseCriteria.getName() + "%"));
			}
			if (!StringUtils.isEmpty(courseCriteria.getCode())) {
				predicates.add(cb.like(root.get("code"), "%" + courseCriteria.getCode() + "%"));
			}
			if (!StringUtils.isEmpty(courseCriteria.getLevel())) {
				predicates.add(cb.equal(root.get("level"), courseCriteria.getLevel()));
			}
			if (!StringUtils.isEmpty(courseCriteria.getEnable())) {
				predicates.add(cb.equal(root.get("enable"), courseCriteria.getEnable()));
			}
			if (specialtyId != null) {
				Subquery<CourseSpeciatly> relationshipSubquery = query
						.subquery(CourseSpeciatly.class);
				Root<CourseSpeciatly> residencyRelationshipSubqueryRoot = relationshipSubquery
						.from(CourseSpeciatly.class);
				relationshipSubquery.select(residencyRelationshipSubqueryRoot);
				Predicate predicate = cb.equal(residencyRelationshipSubqueryRoot.get("specialtyId"),
						specialtyId);
				Predicate predicate2 = cb.equal(residencyRelationshipSubqueryRoot.get("courseId"),
						root.get("id"));
				relationshipSubquery.where(cb.and(predicate, predicate2));
				predicates.add(cb.exists(relationshipSubquery));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	public List<Course> findAll(Course courseCriteria) {
		Specification<Course> specification = getSpecification(courseCriteria, null);
		return courseRepo.findAll(specification);
	}

	public void setCourseEnable(List<Long> courseIds, boolean enable) {
		for (Long courseId : courseIds) {
			Course course = courseRepo.findOne(courseId);
			course.setEnable(enable);
			course.setUpdateTime(new Date());
			courseRepo.saveAndFlush(course);
		}
	}

	@Transactional
	public void delete(String ids) {
		List<Long> courseIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long courseId : courseIds) {
			courseRepo.delete(courseId);
		}
	}

}
